#Cut

#Author:	Eneroth3, eneroth3@gmail.com

#Combability
#	Coded and tested on windows7, Sketchup 8

#Usage
#	Menu: Plugins
#	Toolbar: Cut
#		Hide Cut Surfaces: Move faces and their bounding edges inside groups to the layer 'Cuts' if the face has a material named "!Cut!" or "!Cut#n!" where n is any integer.

#Copyright Julia Christina Eneroth (eneroh3)
#	Free to use both professinal and unprofessional.
#	Free to spread as long as credit goes to me.
#	Please link to my website when publishing online (http://eneroth.port0.org/plugins/cut/).

#Known bugs
#	<None>

#Potential bugs
#	<none>

#Changelog
#	2013-02-24
#		*Recoded almost from skratch.
#		*Cut and reset cuts merged to one function (both moves to and away from cut layer depending on material).
#		*Added toolbar.
#		*Requires cut materials' names to match a pattern ("!Cut!" or "!Cut#n!" where n is any integer) instead of prompting user for their names each time.
#		*Renamed cut layer 'Cuts'.

module Ene_cut

cut_menu = UI.menu("Plugins").add_submenu("Cut")

cut_menu.add_item("Hide Cut Surfaces") { self.cut }
cut_menu.add_separator
cut_menu.add_item("Help") { UI.openURL Sketchup.find_support_file("help.html", "Plugins/ene_cut/") }

tb = UI::Toolbar.new "Cut"

cmd = UI::Command.new("Cut") { self.cut }
cmd.large_icon = cmd.small_icon = "ene_cut/cut.png"
cmd.status_bar_text = cmd.tooltip = "Hide Cut Surfaces"
tb.add_item cmd
	
def self.cut

	Sketchup.status_text = "This operation may take a few minutes..."	#Set stausbar text
	startTime = Time.now.to_i											#Start clock
	
	cutMaterials = []													#Materials at section cuts. Either "!Cut!" or "!Cut#n!" where n is any integer.
	catMaterialPattern = /!Cut(#\d+)?!/
	Sketchup.active_model.materials.each do |i|
		next unless catMaterialPattern.match(i.display_name)
		cutMaterials << i
	end#each
	
	cutLayer = Sketchup.active_model.layers.add "Cuts"					#Create cut layer or return existing
	cutLayer.visible = false
	defaultLayer = Sketchup.active_model.layers[0]
	
	Sketchup.active_model.entities.each do |i|							#Loop model root
	
		next unless i.class == Sketchup::Group							# Skip if not group
		
		i.entities.each do |j|											# Loop group content
		
			isCut = false												#  Set flag. Aassume it is't cut
			if j.class == Sketchup::Face then
				isCut = true if cutMaterials.include? j.material		#  is cut if is face and has cut material
			elsif j.class == Sketchup::Edge then
				j.faces.each do |k|
					if cutMaterials.include? k.material then
						isCut = true									#  is cut if is edge and bounds face with cut material
						break
					end#if
				end#each
			end#if else
			
			if isCut then
				j.layer = cutLayer unless j.layer == cutLayer			#  if is cut, move to cut layer if not already there
			else
				j.layer = defaultLayer if j.layer == cutLayer			#  if is not cut, move to default layer if on cut layer
			end#if else
			
		end#each
	end#each
	
	if  Time.now.to_i-startTime > 10									#Alert user that execution is finished if longer than 10s, tell exectution time
		UI.messagebox "Done. Execution time: " + (Time.now.to_i-startTime).to_s + "s.\n(This notification is shown if execution time is longer than 10s)"
	end#if
	
end#def

end#module